#include <FastLED.h>
#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>
#include <WiFiClient.h>
#include <Arduino_JSON.h>

#define DATA_PIN    D3
#define NUM_LEDS    20
#define BRIGHTNESS  75
#define LED_TYPE    WS2812B
#define COLOR_ORDER GRB

const char* ssid = "REPLACE_WITH_YOUR_SSID"; //<===================================================
const char* password = "REPLACE_WITH_YOUR_PASSWORD";//<===================================================

// Your Domain name with URL path or IP address with path
String openWeatherMapApiKey = "REPLACE_WITH_YOUR_OPEN_WEATHER_MAP_API_KEY";//<===================================================
// Example:
//String openWeatherMapApiKey = "bd939aa3d23ff33d3c8f5dd1dd4";

// Replace with your country code and city
String city = "Bangalore";//<===================================================
String countryCode = "IN";//<===================================================

// THE DEFAULT TIMER IS SET TO 10 SECONDS FOR TESTING PURPOSES
// For a final application, check the API call limits per hour/minute to avoid getting blocked/banned
unsigned long lastTime = 0;
// Timer set to 10 minutes (600000)
//unsigned long timerDelay = 600000;
// Set timer to 10 seconds (10000)
unsigned long timerDelay = 10000;

String jsonBuffer;

float tempr, humid;
float r=0, g=0, b=0;

// Define the array of leds
CRGB leds[NUM_LEDS];

#define UPDATES_PER_SECOND 100

void setup() {
  Serial.begin(115200);

  WiFi.begin(ssid, password);
  Serial.println("Connecting");
  while(WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to WiFi network with IP Address: ");
  Serial.println(WiFi.localIP());
 
  Serial.println("Timer set to 10 seconds (timerDelay variable), it will take 10 seconds before publishing the first reading.");
  FastLED.addLeds<LED_TYPE, DATA_PIN, COLOR_ORDER>(leds, NUM_LEDS);
  FastLED.setBrightness(BRIGHTNESS);
}

void loop() {
  // Send an HTTP GET request
  if ((millis() - lastTime) > timerDelay) {
    // Check WiFi connection status
    if(WiFi.status()== WL_CONNECTED){
      String serverPath = "http://api.openweathermap.org/data/2.5/weather?q=" + city + "," + countryCode + "&APPID=" + openWeatherMapApiKey;
      
      jsonBuffer = httpGETRequest(serverPath.c_str());
      Serial.println(jsonBuffer);
      JSONVar myObject = JSON.parse(jsonBuffer);
  
      // JSON.typeof(jsonVar) can be used to get the type of the var
      if (JSON.typeof(myObject) == "undefined") {
        Serial.println("Parsing input failed!");
        return;
      }
    tempr = int(myObject["main"]["temp"]) - 273.15;
    humid = int(myObject["main"]["humidity"]);
    
    //warm and humid
    if(tempr > 20 && humid > 50)
    {r=100;
    g = 100;
    b=0;
    } // yellow

    // cold and humid
    else if(tempr < 20 && humid > 50)
    {r=0;
    g = 0;
    b=100;
    } //blue

    // warm and dry
    else if(tempr > 20 && humid < 50)
    {r=100;
    g = 0;
    b=0;
    } //red

    //cold and dry
    else
    {r=0;
    g = 100;
    b=100;
    } //cyan

    // Turn the LEDs on
  for(int i=0; i<NUM_LEDS; i++)
  {
  leds[i] = CRGB(r,g,b);
  }
  FastLED.show();
  delay(10*60*1000); // 10 min delay
    
    }
    else {
      Serial.println("WiFi Disconnected");
    }
    lastTime = millis();
  }
}




String httpGETRequest(const char* serverName) {
  WiFiClient client;
  HTTPClient http;
    
  // Your IP address with path or Domain name with URL path 
  http.begin(client, serverName);
  
  // Send HTTP POST request
  int httpResponseCode = http.GET();
  
  String payload = "{}"; 
  
  if (httpResponseCode>0) {
    Serial.print("HTTP Response code: ");
    Serial.println(httpResponseCode);
    payload = http.getString();
  }
  else {
    Serial.print("Error code: ");
    Serial.println(httpResponseCode);
  }

  http.end();

  return payload;
}
